(* 
 * Dusty deBoer, Kansas State University.
 *
 * Based on basicwin.sml, (C) 1990 J.H. Reppy; and goodbye.sml, (C) 1990 AT&T.
 *)

structure XDEMO : sig

    val doit  : string option * string list -> OS.Process.status
    val main  : (string * string list) -> OS.Process.status

  end = struct

    structure EXB = EXeneBase
    structure S = Styles
    structure A = Attrs
    
(* set up the option spec table. *)
val optSpec =  
 [(S.OPT_NAMED("help"), "-help",   S.OPT_NOARG("on"),  A.AT_Bool),
  (S.OPT_NAMED("help"), "-nohelp", S.OPT_NOARG("off"), A.AT_Bool),
  (S.OPT_NAMED("sum"),  "-sum",    S.OPT_ISARG,        A.AT_Str),
  (S.OPT_NAMED("x"),    "-x=",     S.OPT_STICKYARG,    A.AT_Real),
  (S.OPT_NAMED("y"),    "-y",      S.OPT_SEPARG,       A.AT_Real),
  (S.OPT_NAMED("res"),  "-res",    S.OPT_RESARG,       A.AT_Str),
  (S.OPT_NAMED("skip"), "-skip",   S.OPT_SKIPARG,      A.AT_Str),
  (S.OPT_NAMED("ign"),  "-ignore", S.OPT_SKIPLINE,     A.AT_Str),
  (S.OPT_RESSPEC("*background"), "-bg", S.OPT_SEPARG,  A.AT_Str),
  (S.OPT_RESSPEC("*foreground"), "-fg", S.OPT_SEPARG,  A.AT_Str),
  (S.OPT_RESSPEC("*borderColor"),"-bc", S.OPT_SEPARG,  A.AT_Str)]

(* set up application resource defaults. *)
val appResources =
   ["*background: white",
    "*foreground: black"]
    
fun init (dpyOpt,args) =
    let
    val root  = Widget.mkRoot(GetDpy.getDpy(dpyOpt))
        handle EXB.BadAddr s =>
            (TextIO.print s; RunCML.shutdown OS.Process.failure)
            
    (* parse the command line arguments using the option spec table. *)
    val (optDb,unargs) = Widget.parseCommand (optSpec) args
    
    (* obtain the value of a named argument.
     * note that in this case we let the last argument (the head of the returned list)
     * override any previous arguments. *)
    val help  = (case (Widget.findNamedOpt optDb (Styles.OPT_NAMED("help")) root) of
                    [] => false               (* application must supply default here. *)
                  | Attrs.AV_Bool(b)::_ => b) (* let the last argument override. *)
    
    (* obtain the value of a OPT_STICKYARG argument.
     * note that in this case, we use every argument value given. *)
    val sumx = (List.foldl (fn (Attrs.AV_Real(r),s) => (r+s)) 0.0
                (Widget.findNamedOpt optDb (Styles.OPT_NAMED("x")) root)) 
    val sumy = (List.foldl (fn (Attrs.AV_Real(r),s) => (r+s)) 0.0
                (Widget.findNamedOpt optDb (Styles.OPT_NAMED("y")) root))
                
    (* obtain the value of a OPT_ISARG named argument. *)
    val gosum = (case (Widget.findNamedOpt optDb (Styles.OPT_NAMED("sum")) root) of
                    [] => false | Attrs.AV_Str(s)::_ => true)
    
    (* create a style from the application default resource table. *)
    val appStyle = Widget.styleFromStrings(root,appResources)
        handle Styles.PRS.BadSpec (n,s) =>
           (TextIO.print "bad resource specification: ";
            TextIO.print(Int.toString n); TextIO.print (":"^s^"\n");
            Widget.delRoot root; RunCML.shutdown OS.Process.failure)

    (* create a style from the properties stored by xrdb. *)
    val xrdStyle = Widget.styleFromXRDB(root)
        handle Styles.PRS.BadSpec (n,s) =>
           (TextIO.print "bad resource specification: ";
            TextIO.print(Int.toString n); TextIO.print (":"^s^"\n");
            Widget.delRoot root; RunCML.shutdown OS.Process.failure)

    (* create a style from the resource options in the option db. *)
    val argStyle = Widget.styleFromOptDb(root,optDb)
        handle Styles.PRS.BadSpec (n,s) =>
           (TextIO.print "bad resource specification: ";
            TextIO.print(Int.toString n); TextIO.print (":"^s^"\n");
            Widget.delRoot root; RunCML.shutdown OS.Process.failure)

    (* Merge: xrdb strings with app style, overwriting any conflicting app styles.
     * Then merge arg style with the result, giving priority to runtime args. *)
    val mainStyle = Widget.mergeStyles(argStyle,Widget.mergeStyles(xrdStyle,appStyle))
    
    val styleView = Styles.mkView{name = Styles.styleName["demores"],aliases = nil}
    val view      = (styleView, mainStyle)
    
    (* widget setup. *)
    fun quit ()  = (Widget.delRoot root; RunCML.shutdown OS.Process.success)
    
    val quitBttn = Button.textBtn (root, view, [([], Attrs.attr_label, Attrs.AV_Str "Handle Quit")])
    val quitEvt  = Button.evtOf quitBttn
    
    val sumBttn = Button.textBtn (root, view, [([], Attrs.attr_label, Attrs.AV_Str "Sum x+y")])
    val sumEvt  = Button.evtOf sumBttn
    
    val editArgs = []
    val smplEdit1 = SimpleEdit.simpleEdit (root, view, editArgs) (Real.toString sumx)
    val smplEdit2 = SimpleEdit.simpleEdit (root, view, editArgs) (Real.toString sumy)
    val smplEdit3 = SimpleEdit.simpleEdit (root, view, editArgs) ""
    
    fun calcSum () = 
        let
        val r1 = (case (Real.fromString (SimpleEdit.getString smplEdit1)) of
                    SOME r => r | _ => 0.0)
        val r2 = (case (Real.fromString (SimpleEdit.getString smplEdit2)) of
                    SOME r => r | _ => 0.0)
        val rs = Real.toString (r1+r2)
        in SimpleEdit.setString smplEdit3 rs end
    val _ = if (gosum) then calcSum() else ()
    
    val ff1 = FocusFrame.focusframe (root,view,[]) 
        ((SimpleEdit.widgetOf smplEdit1),(SimpleEdit.focusableOf smplEdit1))
    val ff2 = FocusFrame.focusframe (root,view,[]) 
        ((SimpleEdit.widgetOf smplEdit2),(SimpleEdit.focusableOf smplEdit2))
    val ff3 = FocusFrame.focusframe (root,view,[]) 
        ((SimpleEdit.widgetOf smplEdit3),(SimpleEdit.focusableOf smplEdit3))
    
    val lbl1 = Label.label (root, view, [])
    val _    = Label.setLabel lbl1 (Label.Text("sum(x): "))
    val lbl2 = Label.label (root, view, [])
    val _    = Label.setLabel lbl2 (Label.Text("sum(y): "))
    val lbl3 = Label.label (root, view, [])
    val _    = Label.setLabel lbl3 (Label.Text("sum(x)+sum(y): "))
        
    val layout =
        Box.layout (root, view, []) (Box.VtCenter[
            Box.WBox(Button.widgetOf quitBttn),
            Box.WBox(Button.widgetOf sumBttn),
            Box.WBox (Box.widgetOf (Box.layout (root,view,[])
             (Box.HzCenter[
                Box.WBox(Label.widgetOf lbl1),
                Box.WBox(FocusFrame.widgetOf ff1)
            ]))),
            Box.WBox (Box.widgetOf (Box.layout (root,view,[])
             (Box.HzCenter[
                Box.WBox(Label.widgetOf lbl2),
                Box.WBox(FocusFrame.widgetOf ff2)
            ]))),
            Box.WBox (Box.widgetOf (Box.layout (root,view,[])
             (Box.HzCenter[
                Box.WBox(Label.widgetOf lbl3),
                Box.WBox(FocusFrame.widgetOf ff3)
            ])))
        ])
    val shellArgs =
        [([], Attrs.attr_title, Attrs.AV_Str "eXene Resources Demo"),
         ([], Attrs.attr_iconName, Attrs.AV_Str "demo-res")]
    val shell = Shell.shell (root, view, shellArgs) (Box.widgetOf layout)
    
    val id1 = Shell.addFocusableFirst shell (FocusFrame.focusableOf ff1)
    val id2 = Shell.addFocusableAfter shell (id1,FocusFrame.focusableOf ff2)
    val id3 = Shell.addFocusableAfter shell (id2,FocusFrame.focusableOf ff3)
    
    val hints = Shell.mkHints{size_hints=[],wm_hints=[ICCC.HINT_Input(true)]}
    val _     = Shell.setWMHints shell hints
    val cmEvt = Shell.deletionEvent shell 
    
    fun loop():unit =
        let 
        fun handleQuit (Button.BtnUp _)   = (TextIO.print " [demo-res quitting]\n"; quit())
          | handleQuit (_) = (loop()) 
        fun handleSum  (Button.BtnUp (b,t)) = calcSum () 
          | handleSum  (_) = (loop())
        in CML.select
           [CML.wrap(quitEvt, handleQuit),
            CML.wrap(sumEvt,  handleSum),
            CML.wrap(cmEvt, quit)
          ]
        end
    in
        Shell.init shell;
        loop()
    end

  fun doit (dpyOpt,args) = 
      (RunCML.doit (fn () => (init (dpyOpt,args)), NONE))

  fun main (prog, "-display"::(server::args)) = 
            ((TextIO.print ("display="^server)); doit(SOME server,args))
    | main (prog, args) = doit(NONE,args)

end
